/*
 * Copyright (C) 2007 Andre Beckedorf <evilJazz _AT_ katastrophos _DOT_ net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

#ifdef QTOPIA
#include <qpe/qpemenubar.h>
#include <qpe/qpetoolbar.h>
#include <qpe/qcopenvelope_qws.h>
#include <qpe/resource.h>
#include <qpe/global.h>
#else
#include <qmenubar.h>
#include <qtoolbar.h>
#include <resource.h>
// for Qtopia we are using the QPE variants for toolbar and menubar
// to enable special features like the menu key etc.
// Define aliases to avoid ifdefs...
#define QPEMenuBar QMenuBar
#define QPEToolBar QToolBar
#endif

#include <qstringlist.h>
#include <qstring.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qlistview.h>
#include <qspinbox.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qcombobox.h>

#ifdef QT4
#include <QGroupBox>
#include <QStackedWidget>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QTreeWidget>
#include <QHeaderView>
#else
#include <qlistbox.h>
#include <qvgroupbox.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qwidgetstack.h>
#endif

#include <qprogressbar.h>
#include <qtabbar.h>
#include <qdatastream.h>
#include <qmessagebox.h>

#include <qurlinfo.h>
#include <qfile.h>
#include <qregexp.h>

#include "qscrobbler.h"
#include "debug.h"
#include "audioscrobbler.h"
#include "kascachelist.h"
#include "configuration.h"
#include "qscrobblerconfig.h"
#include "qscrobbleraddon.h"

#include "compathack.h"

QScrobbler::QScrobbler(AudioScrobbler *asc, QWidget *parent)
    :	QDialog(parent)
{
#ifndef QTOPIA
#ifdef QT4
#ifdef MAEMO
    setAttribute(Qt::WA_Maemo5StackedWindow);
    setWindowFlags(Qt::Window);
#else
    setWindowFlags(Qt::Window);
    setAttribute(Qt::WA_DeleteOnClose);
#endif
#else
    setWFlags(WType_TopLevel | WStyle_Customize | WStyle_NormalBorder | WStyle_Title | WStyle_SysMenu | WStyle_MinMax | WDestructiveClose);
#endif
#endif

#ifdef QTOPIA
    setWState(WState_Reserved1);
#endif
    setSizeGripEnabled(true);

    setWindowTitle("QScrobbler");

    scrobbler = asc;

#ifdef QT4
    QVBoxLayout *mainLayout = new QVBoxLayout(this);
#ifdef MAEMO
    mainLayout->setMargin(0);
    mainLayout->setSpacing(0);
#else
    mainLayout->setMargin(8);
    mainLayout->setSpacing(-1);
#endif

    QMenu *tools = new QMenu("&Tools", this);
    QAction *pauseSubmissionAct = new QAction(this);
    pauseSubmissionAct->setCheckable(true);
    pauseSubmissionAct->setText(tr("&Pause submissions"));
    connect(pauseSubmissionAct, SIGNAL(toggled(bool)), scrobbler, SLOT(setSubmissionPaused(bool)));
    tools->addAction(pauseSubmissionAct);
    tools->addAction(tr("&Clear list"), this, SLOT(clearScrobbledTracksList()));
    tools->addAction(tr("&Submit now!"), this, SLOT(retrySubmit()));

    QMenu *help = new QMenu("&Help", this);
    help->addAction(tr("&About..."), this, SLOT(about()));

    QMenuBar *menu = new QMenuBar(this);
    menu->addMenu(tools);
    menu->addMenu(help);
    mainLayout->setMenuBar(menu);
#else
    QPopupMenu *help = new QPopupMenu(this);
    help->insertItem(tr("&About..."), this, SLOT(about()));

    QPEMenuBar *menu = new QPEMenuBar(this);
    menu->setMargin(0);
    menu->insertItem("&Help", help);

    QVBoxLayout *mainLayout = new QVBoxLayout(this, 8);
#endif

    // Current Song Groupbox

#ifdef QT4
    gbox1 = new QGroupBox(tr("Song waiting to be scrobbled"), this);
    QVBoxLayout *gbox1Layout = new QVBoxLayout(gbox1);
#else
    gbox1 = new QVGroupBox(tr("Song waiting to be scrobbled"), this);
#endif
    mainLayout->addWidget(gbox1);

    currentlyPlayingLabel = new QLabel(tr("No song."), gbox1);

#ifdef QT4
    gbox1Layout->addWidget(currentlyPlayingLabel);

    progressStack = new QStackedWidget(gbox1);
    gbox1Layout->addWidget(progressStack);
#else
    progressStack = new QWidgetStack(gbox1);
#endif

    progressStack->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
    progressBar = new QProgressBar(progressStack);
    progressStatus = new QLabel(progressStack);
#ifdef QT4
    progressStack->addWidget(progressBar);
    progressStack->addWidget(progressStatus);
#endif

    // Scrobbled Songs Groupbox

#ifdef QT4
    gbox0 = new QGroupBox(tr("Scrobbled songs waiting to be submitted"), this);
    QVBoxLayout *gbox0Layout = new QVBoxLayout(gbox0);
#else
    gbox0 = new QVGroupBox(tr("Scrobbled songs waiting to be submitted"), this);
#endif
    mainLayout->addWidget(gbox0);

#ifdef QT4
    scrobbledTracksListView = new QTreeWidget(gbox0);
    scrobbledTracksListView->setAllColumnsShowFocus(true);
    scrobbledTracksListView->setColumnCount(4);
    scrobbledTracksListView->setHeaderLabels(QStringList()
        << tr("Played at (UTC)")
        << tr("Title")
        << tr("Artist")
        << tr("Album")
    );

    scrobbledTracksListView->setItemsExpandable(false);
    scrobbledTracksListView->setUniformRowHeights(true);
    scrobbledTracksListView->setRootIsDecorated(false);
    scrobbledTracksListView->setWordWrap(false);
    scrobbledTracksListView->setAllColumnsShowFocus(true);
    scrobbledTracksListView->setAlternatingRowColors(true);
    scrobbledTracksListView->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);

    scrobbledTracksListView->header()->setResizeMode(0, QHeaderView::Stretch);
    scrobbledTracksListView->header()->setResizeMode(1, QHeaderView::Stretch);
    scrobbledTracksListView->header()->setResizeMode(2, QHeaderView::Stretch);

    scrobbledTracksListView->header()->setStretchLastSection(false);
    gbox0Layout->addWidget(scrobbledTracksListView);
#else
    scrobbledTracksListView = new QListView(gbox0);
    scrobbledTracksListView->setItemMargin(4);
    scrobbledTracksListView->setAllColumnsShowFocus(true);
    scrobbledTracksListView->setSorting(-1);
    scrobbledTracksListView->setShowSortIndicator(false);

    scrobbledTracksListView->addColumn("Played at (UTC)");
    scrobbledTracksListView->addColumn("Title");
    scrobbledTracksListView->addColumn("Artist");
    scrobbledTracksListView->addColumn("Album");
#endif

#ifndef WINDOWS
    QFont font = scrobbledTracksListView->font();
    font.setPointSize(font.pointSize() * 3 / 4);
    scrobbledTracksListView->setFont(font);
#endif

    QWidget *hbox;

#ifndef MAEMO
#ifdef QT4
    hbox = gbox0;
    QHBoxLayout *hboxLayout = new QHBoxLayout(gbox0);
#else
    hbox = new QHBox(gbox0);
#endif

    QPushButton *pauseSubmissionsButton = new QPushButton("Pause submissions", hbox);
#ifdef QT4
    pauseSubmissionsButton->setCheckable(true);
    pauseSubmissionsButton->setChecked(qScrobblerAddOn.config().submissionPaused());
#else
    pauseSubmissionsButton->setToggleButton(true);
    pauseSubmissionsButton->setOn(qScrobblerAddOn.config().submissionPaused());
#endif
    connect(pauseSubmissionsButton, SIGNAL(toggled(bool)), scrobbler, SLOT(setSubmissionPaused(bool)));

    QPushButton *clearSubmissionsButton = new QPushButton("Clear list", hbox);
    connect(clearSubmissionsButton, SIGNAL(clicked()), this, SLOT(clearScrobbledTracksList()));

    QPushButton *retrySubmissionButton = new QPushButton("Submit now!", hbox);
    connect(retrySubmissionButton, SIGNAL(clicked()), this, SLOT(retrySubmit()));

#ifdef QT4
    gbox0Layout->addLayout(hboxLayout);
    hboxLayout->addWidget(pauseSubmissionsButton);
    hboxLayout->addWidget(clearSubmissionsButton);
    hboxLayout->addWidget(retrySubmissionButton);
#endif
#endif

    // Status Groupbox

#ifdef QT4
    QGroupBox *gbox2 = new QGroupBox(tr("Status:"), this);
    hbox = gbox2;
    QHBoxLayout *gbox2Layout = new QHBoxLayout(gbox2);
#else
    QVGroupBox *gbox2 = new QVGroupBox(tr("Status:"), this);
    hbox = new QHBox(gbox2);
#endif
    mainLayout->addWidget(gbox2);
    statusLabel = new QLabel("ready.", hbox);

#ifdef QT4
    gbox2Layout->addWidget(statusLabel);
    // TODO last.fm logo badge...
#else
    QPixmap badgePixmap = Resource::loadPixmap("lastfm");
    if (!badgePixmap.isNull())
    {
        QWidget *badge = new QWidget(hbox);
        badge->setBackgroundPixmap(badgePixmap);
        if (badge->backgroundPixmap())
            badge->setFixedSize(badge->backgroundPixmap()->width(), badge->backgroundPixmap()->height());
    }
#endif

    connect(scrobbler, SIGNAL(statusMessage(const QString&)), statusLabel, SLOT(setText(const QString&)));
    connect(scrobbler, SIGNAL(cacheSaved()), this, SLOT(cacheSaved()));
    connect(scrobbler, SIGNAL(songAddedForSubmission(const QString&)), this, SLOT(songAddedForSubmission(const QString&)));
    connect(scrobbler, SIGNAL(songRemoved()), this, SLOT(removeCurrentSong()));

    connect(scrobbler, SIGNAL(songAdded(const QString &, const QString &, const QString &, const QString &, unsigned int)),
            this, SLOT(songAdded(const QString &, const QString &, const QString &, const QString &, unsigned int)));

    updateTimer = new QTimer();
    connect(updateTimer, SIGNAL(timeout()), this, SLOT(updateTimerTriggered()));

    if (scrobbler->isActive())
    {
        songAdded(scrobbler->currentSong().artist, scrobbler->currentSong().songtitle, scrobbler->currentSong().album, scrobbler->currentSong().mbid, scrobbler->currentSong().length);
    }
    else
    {
        removeCurrentSong();
    }

    cacheSaved();
}

QScrobbler::~QScrobbler()
{
    disconnect(scrobbler, SIGNAL(statusMessage(const QString&)), statusLabel, SLOT(setText(const QString&)));
    disconnect(scrobbler, SIGNAL(cacheSaved()), this, SLOT(cacheSaved()));
    disconnect(scrobbler, SIGNAL(songAddedForSubmission(const QString&)), this, SLOT(songAddedForSubmission(const QString&)));
    disconnect(scrobbler, SIGNAL(songRemoved()), this, SLOT(removeCurrentSong()));
    disconnect(scrobbler, SIGNAL(songAdded(const QString &, const QString &, const QString &, const QString &, unsigned int)),
               this, SLOT(songAdded(const QString &, const QString &, const QString &, const QString &, unsigned int)));
}

void QScrobbler::closeEvent(QCloseEvent *e)
{
    emit closing();
    e->accept();
}

void QScrobbler::done(int r)
{
    emit closing();
    QDialog::done(r);
}

void QScrobbler::showEvent(QShowEvent *)
{
#ifdef QTOPIA
    showMaximized();
#endif;
}

void QScrobbler::cacheSaved()
{
    DENTERMETHOD;

    int subCount = scrobbler->cache().submissionCount();

    scrobbledTracksListView->setUpdatesEnabled(false);
    scrobbledTracksListView->blockSignals(true);
    scrobbledTracksListView->clear();

    for (int i = 0; i < subCount; i++)
    {
        const ScrobSongData songData = scrobbler->cache().getSubmission(i);
#ifdef QT4
        QTreeWidgetItem *li = new QTreeWidgetItem(scrobbledTracksListView);
#else
        QListViewItem *li = new QListViewItem(scrobbledTracksListView);
#endif
        li->setText(0, songData.time);
        li->setText(1, songData.songtitle);
        li->setText(2, songData.artist);
        li->setText(3, songData.album);
    }

#ifdef QT4
    if (scrobbledTracksListView->topLevelItem(0))
    {
        scrobbledTracksListView->setCurrentItem(scrobbledTracksListView->topLevelItem(0));
        scrobbledTracksListView->topLevelItem(0)->setSelected(true);
    }
#else
    if (scrobbledTracksListView->firstChild())
    {
        scrobbledTracksListView->setCurrentItem(scrobbledTracksListView->firstChild());
        scrobbledTracksListView->setSelected(scrobbledTracksListView->firstChild(), true);
    }
#endif

    scrobbledTracksListView->blockSignals(false);
    scrobbledTracksListView->setUpdatesEnabled(true);
#ifdef QT4
    scrobbledTracksListView->repaint();
#else
    scrobbledTracksListView->triggerUpdate();
#endif

    gbox0->setTitle(tr("Scrobbled songs waiting to be submitted") + ": " + QString::number(subCount));

    DEXITMETHOD;
}

void QScrobbler::clearScrobbledTracksList()
{
    int ret = QMessageBox::warning(this, tr("Confirmation"),
                                   tr("Do you really want to clear all scrobbled tracks?"),
                                   tr("Yes"), tr("No"), QString::null,
                                   1, 1);

    if (ret == 0)
        scrobbler->clearCache();
}

void QScrobbler::retrySubmit()
{
#ifdef MAEMO
    if (!QMaemoInternetConnectivity::isConnected())
        QMaemoInternetConnectivity::connectionRequest();
#endif
    scrobbler->reinitialize();
}

void QScrobbler::songAddedForSubmission(const QString& songTitle)
{
    DENTERMETHOD;

    gbox1->setTitle(tr("Song currently playing"));
    progressStatus->setText(tr("Song successfully scrobbled."));

#ifdef QT4
    progressStack->setCurrentWidget(progressStatus);
    progressBar->setValue(100);
#else
    progressStack->raiseWidget(progressStatus);
    progressBar->setProgress(100);
#endif
    updateTimer->stop();

    DEXITMETHOD;
}

void QScrobbler::about()
{
    QMessageBox mb("About QScrobbler",
                   "QScrobbler " + QString("%1.%2").arg(APPVERSION_MAJOR).arg(APPVERSION_MINOR) + "\n"
                   "Copyright (C) 2007-2011 by Andre Beckedorf\n"
                   "evilJazz@katastrophos.net\n"
                   "http://katastrophos.net/qscrobbler\n\n"
                   "QScrobbler is based on portions of KAudioScrobbler\n"
                   "Copyright (C) 2004 by Steven Scott\n"
                   "http://code.google.com/p/kaudioscrobbler/\n\n"
                   "This software is licensed under the terms of\n"
                   "the GNU Public License version 2.", QMessageBox::NoIcon, 1, 0, 0);
    mb.exec();
}

void QScrobbler::updateTimerTriggered()
{
    DENTERMETHOD;

    qDebug("scrobbler->requiredTime(): %d, scrobbler->currentRemainingTime(): %d", scrobbler->requiredTime(), scrobbler->currentRemainingTime());

    if (scrobbler->currentRemainingTime() <= 0)
    {
#ifdef QT4
        progressBar->setValue(scrobbler->requiredTime());
#else
        progressBar->setProgress(scrobbler->requiredTime());
#endif
        updateTimer->stop();
    }
    else
    {
#ifdef QT4
        progressBar->setValue(scrobbler->requiredTime() - scrobbler->currentRemainingTime());
#else
        progressBar->setProgress(scrobbler->requiredTime() - scrobbler->currentRemainingTime());
#endif
    }

    statusLabel->setText(scrobbler->lastStatus());

    DEXITMETHOD;
}

void QScrobbler::songAdded(const QString &artist, const QString &title, const QString &album, const QString &musicbrainzid, unsigned int length)
{
    DENTERMETHOD;

    gbox1->setTitle(tr("Song waiting to be scrobbled"));
    currentlyPlayingLabel->setText(artist + " - " + title);
#ifdef QT4
    progressBar->setMaximum(scrobbler->requiredTime());
    progressStack->setCurrentWidget(progressBar);
    progressBar->setValue(0);
#else
    progressBar->setTotalSteps(scrobbler->requiredTime());
    progressStack->raiseWidget(progressBar);
    progressBar->setProgress(0);
#endif

    updateTimer->start(3000);
    updateTimerTriggered();

    DEXITMETHOD;
}

void QScrobbler::removeCurrentSong()
{
    DPRINTF("stopped...");
    gbox1->setTitle(tr("Song waiting to be scrobbled"));
    currentlyPlayingLabel->setText(tr("No song."));
    progressStatus->setText("");
#ifdef QT4
    progressStack->setCurrentWidget(progressStatus);
#else
    progressStack->raiseWidget(progressStatus);
#endif

    updateTimer->stop();
}
